@@ -1,3 +1,5 @@
+#![allow(clippy::type_complexity)]
+
 use std::{
     fmt,
     future::Future,
@@ -186,7 +188,7 @@ impl crate::RenderInner<Context> for RenderPass {
         self.0
             .set_vertex_buffer_with_f64_and_f64(slot, &buffer.0, offset as f64, mapped_size);
     }
-    fn set_push_constants(&mut self, _stages: wgt::ShaderStage, _offset: u32, _data: &[u8]) {
+    fn set_push_constants(&mut self, _stages: wgt::ShaderStages, _offset: u32, _data: &[u8]) {
         panic!("PUSH_CONSTANTS feature must be enabled to call multi_draw_indexed_indirect")
     }
     fn draw(&mut self, vertices: Range<u32>, instances: Range<u32>) {
@@ -315,7 +317,7 @@ impl crate::RenderInner<Context> for RenderBundleEncoder {
         self.0
             .set_vertex_buffer_with_f64_and_f64(slot, &buffer.0, offset as f64, mapped_size);
     }
-    fn set_push_constants(&mut self, _stages: wgt::ShaderStage, _offset: u32, _data: &[u8]) {
+    fn set_push_constants(&mut self, _stages: wgt::ShaderStages, _offset: u32, _data: &[u8]) {
         panic!("PUSH_CONSTANTS feature must be enabled to call multi_draw_indexed_indirect")
     }
     fn draw(&mut self, vertices: Range<u32>, instances: Range<u32>) {
@@ -752,12 +754,12 @@ fn map_vertex_format(format: wgt::VertexFormat) -> web_sys::GpuVertexFormat {
     }
 }
 
-fn map_input_step_mode(mode: wgt::InputStepMode) -> web_sys::GpuInputStepMode {
+fn map_input_step_mode(mode: wgt::VertexStepMode) -> web_sys::GpuInputStepMode {
     use web_sys::GpuInputStepMode as sm;
-    use wgt::InputStepMode;
+    use wgt::VertexStepMode;
     match mode {
-        InputStepMode::Vertex => sm::Vertex,
-        InputStepMode::Instance => sm::Instance,
+        VertexStepMode::Vertex => sm::Vertex,
+        VertexStepMode::Instance => sm::Instance,
     }
 }
 
@@ -907,9 +909,8 @@ impl crate::Context for Context {
     type RenderBundleEncoderId = RenderBundleEncoder;
     type RenderBundleId = Sendable<web_sys::GpuRenderBundle>;
     type SurfaceId = Sendable<web_sys::GpuCanvasContext>;
-    type SwapChainId = Sendable<web_sys::GpuSwapChain>;
 
-    type SwapChainOutputDetail = SwapChainOutputDetail;
+    type SurfaceOutputDetail = SurfaceOutputDetail;
 
     type RequestAdapterFuture = MakeSendFuture<
         wasm_bindgen_futures::JsFuture,
@@ -923,8 +924,10 @@ impl crate::Context for Context {
         wasm_bindgen_futures::JsFuture,
         fn(JsFutureResult) -> Result<(), crate::BufferAsyncError>,
     >;
+    type OnSubmittedWorkDoneFuture =
+        MakeSendFuture<wasm_bindgen_futures::JsFuture, fn(JsFutureResult) -> ()>;
 
-    fn init(_backends: wgt::BackendBit) -> Self {
+    fn init(_backends: wgt::Backends) -> Self {
         Context(web_sys::window().unwrap().navigator().gpu())
     }
 
@@ -953,6 +956,14 @@ impl crate::Context for Context {
         Sendable(context.into())
     }
 
+    fn adapter_is_surface_supported(
+        &self,
+        _adapter: &Self::AdapterId,
+        _surface: &Self::SurfaceId,
+    ) -> bool {
+        true
+    }
+
     fn instance_request_adapter(
         &self,
         options: &crate::RequestAdapterOptions<'_>,
@@ -960,7 +971,7 @@ impl crate::Context for Context {
         //TODO: support this check, return `None` if the flag is not set.
         // It's not trivial, since we need the Future logic to have this check,
         // and currently the Future here has no room for extra parameter `backends`.
-        //assert!(backends.contains(wgt::BackendBit::BROWSER_WEBGPU));
+        //assert!(backends.contains(wgt::Backends::BROWSER_WEBGPU));
         let mut mapped_options = web_sys::GpuRequestAdapterOptions::new();
         let mapped_power_preference = match options.power_preference {
             wgt::PowerPreference::LowPower => web_sys::GpuPowerPreference::LowPower,
@@ -990,11 +1001,6 @@ impl crate::Context for Context {
         if trace_dir.is_some() {
             //Error: Tracing isn't supported on the Web target
         }
-        assert!(
-            !desc.features.intersects(crate::Features::ALL_NATIVE),
-            "The web backend doesn't support any native extensions. Enabled native extensions: {:?}",
-            desc.features & crate::Features::ALL_NATIVE
-        );
 
         // TODO: non-guaranteed limits
         let mut mapped_desc = web_sys::GpuDeviceDescriptor::new();
@@ -1038,16 +1044,6 @@ impl crate::Context for Context {
         )
     }
 
-    fn adapter_get_swap_chain_preferred_format(
-        &self,
-        adapter: &Self::AdapterId,
-        surface: &Self::SurfaceId,
-    ) -> Option<wgt::TextureFormat> {
-        let format =
-            map_texture_format_from_web_sys(surface.0.get_swap_chain_preferred_format(&adapter.0));
-        Some(format)
-    }
-
     fn adapter_features(&self, adapter: &Self::AdapterId) -> wgt::Features {
         // TODO
         let _features = adapter.0.features();
@@ -1080,9 +1076,12 @@ impl crate::Context for Context {
         }
     }
 
-    fn adapter_downlevel_properties(&self, _adapter: &Self::AdapterId) -> wgt::DownlevelProperties {
+    fn adapter_downlevel_properties(
+        &self,
+        _adapter: &Self::AdapterId,
+    ) -> wgt::DownlevelCapabilities {
         // WebGPU is assumed to be fully compliant
-        wgt::DownlevelProperties::default()
+        wgt::DownlevelCapabilities::default()
     }
 
     fn adapter_get_info(&self, _adapter: &Self::AdapterId) -> wgt::AdapterInfo {
@@ -1104,6 +1103,47 @@ impl crate::Context for Context {
         format.describe().guaranteed_format_features
     }
 
+    fn surface_get_preferred_format(
+        &self,
+        surface: &Self::SurfaceId,
+        adapter: &Self::AdapterId,
+    ) -> Option<wgt::TextureFormat> {
+        let format =
+            map_texture_format_from_web_sys(surface.0.get_swap_chain_preferred_format(&adapter.0));
+        Some(format)
+    }
+
+    fn surface_configure(
+        &self,
+        surface: &Self::SurfaceId,
+        device: &Self::DeviceId,
+        config: &wgt::SurfaceConfiguration,
+    ) {
+        let mut mapped =
+            web_sys::GpuSwapChainDescriptor::new(&device.0, map_texture_format(config.format));
+        mapped.usage(config.usage.bits());
+        surface.0.configure_swap_chain(&mapped);
+    }
+
+    fn surface_get_current_texture(
+        &self,
+        _surface: &Self::SurfaceId,
+    ) -> (
+        Option<Self::TextureId>,
+        wgt::SurfaceStatus,
+        Self::SurfaceOutputDetail,
+    ) {
+        (
+            None, //TODO: surface.0.get_current_texture(),
+            wgt::SurfaceStatus::Good,
+            (),
+        )
+    }
+
+    fn surface_present(&self, _texture: &Self::TextureId, _detail: &Self::SurfaceOutputDetail) {
+        // Swapchain is presented automatically
+    }
+
     fn device_features(&self, _device: &Self::DeviceId) -> wgt::Features {
         // TODO
         wgt::Features::empty()
@@ -1114,21 +1154,9 @@ impl crate::Context for Context {
         wgt::Limits::default()
     }
 
-    fn device_downlevel_properties(&self, _device: &Self::DeviceId) -> wgt::DownlevelProperties {
+    fn device_downlevel_properties(&self, _device: &Self::DeviceId) -> wgt::DownlevelCapabilities {
         // WebGPU is assumed to be fully compliant
-        wgt::DownlevelProperties::default()
-    }
-
-    fn device_create_swap_chain(
-        &self,
-        device: &Self::DeviceId,
-        surface: &Self::SurfaceId,
-        desc: &wgt::SwapChainDescriptor,
-    ) -> Self::SwapChainId {
-        let mut mapped =
-            web_sys::GpuSwapChainDescriptor::new(&device.0, map_texture_format(desc.format));
-        mapped.usage(desc.usage.bits());
-        Sendable(surface.0.configure_swap_chain(&mapped))
+        wgt::DownlevelCapabilities::default()
     }
 
     fn device_create_shader_module(
@@ -1137,28 +1165,15 @@ impl crate::Context for Context {
         desc: &crate::ShaderModuleDescriptor,
     ) -> Self::ShaderModuleId {
         let mut descriptor = match desc.source {
+            #[cfg(feature = "spirv")]
             crate::ShaderSource::SpirV(ref spv) => {
                 web_sys::GpuShaderModuleDescriptor::new(&js_sys::Uint32Array::from(&**spv))
             }
             crate::ShaderSource::Wgsl(ref code) => {
-                use naga::{back::spv, front::wgsl, valid::Validator};
-                let module = wgsl::parse_str(code).unwrap();
-                let options = spv::Options {
-                    lang_version: (1, 0),
-                    flags: spv::WriterFlags::empty(),
-                    capabilities: None,
-                };
-                let analysis = Validator::new(
-                    naga::valid::ValidationFlags::empty(),
-                    naga::valid::Capabilities::all(),
-                )
-                .validate(&module)
-                .unwrap();
-                let words = spv::write_vec(&module, &analysis, &options).unwrap();
-                web_sys::GpuShaderModuleDescriptor::new(&js_sys::Uint32Array::from(&words[..]))
+                web_sys::GpuShaderModuleDescriptor::new(&js_sys::JsString::from(&**code))
             }
         };
-        if let Some(ref label) = desc.label {
+        if let Some(label) = desc.label {
             descriptor.label(label);
         }
         Sendable(device.0.create_shader_module(&descriptor))
@@ -1229,12 +1244,12 @@ impl crate::Context for Context {
                         view_dimension,
                     } => {
                         let mapped_access = match access {
-                            wgt::StorageTextureAccess::ReadOnly => {
-                                web_sys::GpuStorageTextureAccess::ReadOnly
-                            }
                             wgt::StorageTextureAccess::WriteOnly => {
                                 web_sys::GpuStorageTextureAccess::WriteOnly
                             }
+                            wgt::StorageTextureAccess::ReadOnly => {
+                                panic!("ReadOnly is not available")
+                            }
                             wgt::StorageTextureAccess::ReadWrite => {
                                 panic!("ReadWrite is not available")
                             }
@@ -1253,12 +1268,20 @@ impl crate::Context for Context {
             .collect::<js_sys::Array>();
 
         let mut mapped_desc = web_sys::GpuBindGroupLayoutDescriptor::new(&mapped_bindings);
-        if let Some(ref label) = desc.label {
+        if let Some(label) = desc.label {
             mapped_desc.label(label);
         }
         Sendable(device.0.create_bind_group_layout(&mapped_desc))
     }
 
+    unsafe fn device_create_shader_module_spirv(
+        &self,
+        _device: &Self::DeviceId,
+        _desc: &crate::ShaderModuleDescriptorSpirV,
+    ) -> Self::ShaderModuleId {
+        unreachable!("SPIRV_SHADER_PASSTHROUGH is not enabled for this backend")
+    }
+
     fn device_create_bind_group(
         &self,
         device: &Self::DeviceId,
@@ -1270,7 +1293,7 @@ impl crate::Context for Context {
             .map(|binding| {
                 let mapped_resource = match binding.resource {
                     crate::BindingResource::Buffer(crate::BufferBinding {
-                        ref buffer,
+                        buffer,
                         offset,
                         size,
                     }) => {
@@ -1280,15 +1303,13 @@ impl crate::Context for Context {
                         if let Some(s) = size {
                             mapped_buffer_binding.size(s.get() as f64);
                         }
-                        JsValue::from(mapped_buffer_binding.clone())
+                        JsValue::from(mapped_buffer_binding)
                     }
                     crate::BindingResource::BufferArray(..) => {
                         panic!("Web backend does not support arrays of buffers")
                     }
-                    crate::BindingResource::Sampler(ref sampler) => {
-                        JsValue::from(sampler.id.0.clone())
-                    }
-                    crate::BindingResource::TextureView(ref texture_view) => {
+                    crate::BindingResource::Sampler(sampler) => JsValue::from(sampler.id.0.clone()),
+                    crate::BindingResource::TextureView(texture_view) => {
                         JsValue::from(texture_view.id.0.clone())
                     }
                     crate::BindingResource::TextureViewArray(..) => {
@@ -1331,7 +1352,7 @@ impl crate::Context for Context {
         desc: &crate::RenderPipelineDescriptor,
     ) -> Self::RenderPipelineId {
         let mut mapped_vertex_state =
-            web_sys::GpuVertexState::new(&desc.vertex.entry_point, &desc.vertex.module.id.0);
+            web_sys::GpuVertexState::new(desc.vertex.entry_point, &desc.vertex.module.id.0);
 
         let buffers = desc
             .vertex
@@ -1393,7 +1414,7 @@ impl crate::Context for Context {
                 })
                 .collect::<js_sys::Array>();
             let mapped_fragment_desc =
-                web_sys::GpuFragmentState::new(&frag.entry_point, &frag.module.id.0, &targets);
+                web_sys::GpuFragmentState::new(frag.entry_point, &frag.module.id.0, &targets);
             mapped_desc.fragment(&mapped_fragment_desc);
         }
 
@@ -1415,7 +1436,7 @@ impl crate::Context for Context {
         desc: &crate::ComputePipelineDescriptor,
     ) -> Self::ComputePipelineId {
         let mapped_compute_stage =
-            web_sys::GpuProgrammableStage::new(&desc.entry_point, &desc.module.id.0);
+            web_sys::GpuProgrammableStage::new(desc.entry_point, &desc.module.id.0);
         let mut mapped_desc = web_sys::GpuComputePipelineDescriptor::new(&mapped_compute_stage);
         if let Some(layout) = desc.layout {
             mapped_desc.layout(&layout.id.0);
@@ -1434,7 +1455,7 @@ impl crate::Context for Context {
         let mut mapped_desc =
             web_sys::GpuBufferDescriptor::new(desc.size as f64, desc.usage.bits());
         mapped_desc.mapped_at_creation(desc.mapped_at_creation);
-        if let Some(ref label) = desc.label {
+        if let Some(label) = desc.label {
             mapped_desc.label(label);
         }
         Sendable(device.0.create_buffer(&mapped_desc))
@@ -1450,7 +1471,7 @@ impl crate::Context for Context {
             &map_extent_3d(desc.size),
             desc.usage.bits(),
         );
-        if let Some(ref label) = desc.label {
+        if let Some(label) = desc.label {
             mapped_desc.label(label);
         }
         mapped_desc.dimension(map_texture_dimension(desc.dimension));
@@ -1487,9 +1508,8 @@ impl crate::Context for Context {
     fn device_create_query_set(
         &self,
         _device: &Self::DeviceId,
-        _desc: &wgt::QuerySetDescriptor,
+        _desc: &wgt::QuerySetDescriptor<crate::Label>,
     ) -> Self::QuerySetId {
-        ()
     }
 
     fn device_create_command_encoder(
@@ -1498,7 +1518,7 @@ impl crate::Context for Context {
         desc: &crate::CommandEncoderDescriptor,
     ) -> Self::CommandEncoderId {
         let mut mapped_desc = web_sys::GpuCommandEncoderDescriptor::new();
-        if let Some(ref label) = desc.label {
+        if let Some(label) = desc.label {
             mapped_desc.label(label);
         }
         device
@@ -1517,11 +1537,11 @@ impl crate::Context for Context {
             .map(|cf| wasm_bindgen::JsValue::from(map_texture_format(*cf)))
             .collect::<js_sys::Array>();
         let mut mapped_desc = web_sys::GpuRenderBundleEncoderDescriptor::new(&mapped_color_formats);
-        if let Some(ref label) = desc.label {
+        if let Some(label) = desc.label {
             mapped_desc.label(label);
         }
-        if let Some(dsf) = desc.depth_stencil_format {
-            mapped_desc.depth_stencil_format(map_texture_format(dsf));
+        if let Some(ds) = desc.depth_stencil {
+            mapped_desc.depth_stencil_format(map_texture_format(ds.format));
         }
         mapped_desc.sample_count(desc.sample_count);
         RenderBundleEncoder(device.0.create_render_bundle_encoder(&mapped_desc))
@@ -1582,31 +1602,6 @@ impl crate::Context for Context {
         buffer.0.unmap();
     }
 
-    fn swap_chain_get_current_texture_view(
-        &self,
-        swap_chain: &Self::SwapChainId,
-    ) -> (
-        Option<Self::TextureViewId>,
-        wgt::SwapChainStatus,
-        Self::SwapChainOutputDetail,
-    ) {
-        // TODO: Should we pass a descriptor here?
-        // Or is the default view always correct?
-        (
-            Some(Sendable(swap_chain.0.get_current_texture().create_view())),
-            wgt::SwapChainStatus::Good,
-            (),
-        )
-    }
-
-    fn swap_chain_present(
-        &self,
-        _view: &Self::TextureViewId,
-        _detail: &Self::SwapChainOutputDetail,
-    ) {
-        // Swapchain is presented automatically
-    }
-
     fn texture_create_view(
         &self,
         texture: &Self::TextureId,
@@ -1788,7 +1783,7 @@ impl crate::Context for Context {
         desc: &crate::ComputePassDescriptor,
     ) -> Self::ComputePassId {
         let mut mapped_desc = web_sys::GpuComputePassDescriptor::new();
-        if let Some(ref label) = desc.label {
+        if let Some(label) = desc.label {
             mapped_desc.label(label);
         }
         ComputePass(encoder.begin_compute_pass_with_descriptor(&mapped_desc))
@@ -2043,11 +2038,18 @@ impl crate::Context for Context {
         1.0 //TODO
     }
 
+    fn queue_on_submitted_work_done(
+        &self,
+        _queue: &Self::QueueId,
+    ) -> Self::OnSubmittedWorkDoneFuture {
+        unimplemented!()
+    }
+
     fn device_start_capture(&self, _device: &Self::DeviceId) {}
     fn device_stop_capture(&self, _device: &Self::DeviceId) {}
 }
 
-pub(crate) type SwapChainOutputDetail = ();
+pub(crate) type SurfaceOutputDetail = ();
 
 #[derive(Debug)]
 pub struct BufferMappedRange {
